﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Logging
{


    /// <summary>
    /// A PDFTraceLog implementation that sends the same message to multiple child logs
    /// </summary>
    public class CompositeTraceLog : PDFTraceLog
    {

        private IEnumerable<PDFTraceLog> _inner;

        public CompositeTraceLog(TraceRecordLevel level, IEnumerable<PDFTraceLog> entries)
            : base(level)
        {
            if (null == entries)
                throw new ArgumentNullException("entries");
            this._inner = entries;
        }

        internal protected override void Record(string inset, TraceLevel level, TimeSpan timestamp, string category, string message, Exception ex)
        {
            foreach (PDFTraceLog log in _inner)
            {
                if (log.ShouldLog(level))
                    log.Record(inset, level, timestamp, category, message, ex);
            }

        }
    }
}
