﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Scryber
{
    /// <summary>
    /// The context class for databinding
    /// </summary>
    public class PDFDataContext : PDFContextBase
    {

        #region public PDFDataStack DataStack {get;}

        private PDFDataStack _datastack;
        
        /// <summary>
        /// Gets the current DataStack that holds the objects to bind to.
        /// </summary>
        public PDFDataStack DataStack
        {
            get { return _datastack; }
        }

        #endregion

        #region  public int CurrentIndex {get;set;}

        private int _currindex;

        /// <summary>
        /// Gets or sets the current databinding index
        /// </summary>
        public int CurrentIndex
        {
            get { return _currindex; }
            set { _currindex = value; }
        }

        #endregion

        #region public System.Xml.Xsl.XsltContext NamespaceResolver {get;set;}

        private System.Xml.IXmlNamespaceResolver _xslt;

        /// <summary>
        /// Gets or setsthe XSLT Context for this data context. 
        /// The XsltContext can be used to provide custom functions and values duing processing
        /// </summary>
        public System.Xml.IXmlNamespaceResolver NamespaceResolver
        {
            get { return _xslt; }
            set { _xslt = value; }
        }

        #endregion

        //
        // .ctors
        //

        #region public PDFDataContext(PDFItemCollection items, PDFTraceLog log)

        /// <summary>
        /// Creates a new PDFDataContext with the item collection and trace log
        /// </summary>
        /// <param name="items"></param>
        /// <param name="log"></param>
        public PDFDataContext(PDFItemCollection items, PDFTraceLog log)
            : this(items, log, new PDFDataStack())
        {
        }

        #endregion

        #region public PDFDataContext(PDFItemCollection items, PDFTraceLog log, PDFDataStack stack)

        /// <summary>
        /// Creates a new PDFDataContext with the item collection, trace log, and data stack
        /// </summary>
        /// <param name="items"></param>
        /// <param name="log"></param>
        /// <param name="stack"></param>
        public PDFDataContext(PDFItemCollection items, PDFTraceLog log, PDFDataStack stack)
            : base(items, log)
        {
            this._datastack = stack;
        }

        #endregion

        //
        // legacy
        //

        //public object Eval(string expression)
        //{
        //    if (this.DataStack.HasData)
        //        return PDFDataHelper.Evaluate(this.DataStack.Current, expression);
        //    else
        //        return null;
        //}

        //public System.Xml.XPath.XPathNavigator EvalXPath(System.Xml.XPath.XPathExpression expr)
        //{
        //    if (this.DataStack.HasData)
        //    {
        //        if (this.DataStack.Current is System.Xml.XPath.IXPathNavigable)
        //        {
        //            System.Xml.XPath.IXPathNavigable nav = this.DataStack.Current as System.Xml.XPath.IXPathNavigable;
        //            return nav.CreateNavigator().Select(expr).Current;
        //        }
        //        else
        //            throw new ArgumentException(Errors.DatabindingSourceNotXPath, "expr");
        //    }
        //    else
        //        return null;
        //}

        //public string EvalXPathAsString(System.Xml.XPath.XPathExpression expr)
        //{
        //    System.Xml.XPath.XPathNavigator nav = (System.Xml.XPath.XPathNavigator)this.DataStack.Current;
        //    string result;
        //    if (null == nav)
        //        return string.Empty;
        //    else
        //    {
        //        switch (expr.ReturnType)
        //        {
        //            case System.Xml.XPath.XPathResultType.Boolean:
        //                bool value = (bool)nav.Evaluate(expr);
        //                result = value.ToString();
        //                break;

        //            case System.Xml.XPath.XPathResultType.NodeSet:
        //                nav = nav.SelectSingleNode(expr);
        //                if (null == nav)
        //                    result = string.Empty;
        //                else
        //                    result = nav.Value;
        //                break;

        //            case System.Xml.XPath.XPathResultType.Number:
        //                result = nav.Evaluate(expr).ToString();
        //                break;

        //            case System.Xml.XPath.XPathResultType.String: //Includes navigator!!!!
        //                result = nav.Evaluate(expr).ToString();
        //                break;

        //            case System.Xml.XPath.XPathResultType.Error:
        //                throw new ArgumentException(String.Format(Errors.XPathExpressionCouldNotBeEvaluated, expr.Expression), "expr");

        //            case System.Xml.XPath.XPathResultType.Any:
        //            default:
        //                result = nav.Evaluate(expr).ToString();
        //                break;
        //        }
        //    }

        //    return result;
        //}

        

        //public System.Xml.XPath.XPathNavigator EvalXPath(string xpath)
        //{
        //    if (this.DataStack.HasData)
        //    {
        //        if (this.DataStack.Current is System.Xml.XPath.IXPathNavigable)
        //        {
        //            System.Xml.XPath.IXPathNavigable nav = this.DataStack.Current as System.Xml.XPath.IXPathNavigable;
        //            return nav.CreateNavigator().Select(xpath).Current;
        //        }
        //        else
        //            throw new ArgumentException(Errors.DatabindingSourceNotXPath, "xpath");
        //    }
        //    else
        //        return null;
        //}

    }

    
}
