﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Scryber.Styles;
using Scryber.Drawing;

namespace Scryber.Components.Layout
{
    public class PanelLayoutEngine : PDFBlockLayoutEngineBase
    {
        


        //
        // ctor
        //

        #region protected PanelLayoutEngine(PDFContainerComponent container, PDFLayoutContext context, PDFStyle fullstyle)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="container"></param>
        /// <param name="context"></param>
        /// <param name="fullstyle"></param>
        public PanelLayoutEngine(PDFContainerComponent container, IPDFLayoutEngine parent)
            : base(container, parent)
        {
        }


        #endregion



        protected override void DoLayoutComponent()
        {
            if (this.TraceLog.ShouldLog(TraceLevel.Debug))
                this.TraceLog.Begin(TraceLevel.Debug, "Panel Layout Engine", "Beginning layout of component '" + this.Component + "{0}' as a viewport component");

            PDFPositionOptions pos = this.FullStyle.CreatePostionOptions();
            if (pos.PositionMode != Drawing.PositionMode.Inline)
            {
                this.DoLayoutBlockComponent(pos);
            }
            else
            {
                this.DoLayoutChildren();
            }

            if (this.TraceLog.ShouldLog(TraceLevel.Debug))
                this.TraceLog.End(TraceLevel.Debug, "Panel Layout Engine", "Completed layout of component '" + this.Component + "{0}' as a viewport component");

        }

        


        /// <summary>
        /// Lays out all the content of this panel
        /// </summary>
        /// <param name="position"></param>
        private void DoLayoutBlockComponent(PDFPositionOptions position)
        {
            bool newPage = false;
            PDFLayoutBlock containerBlock = this.DocumentLayout.CurrentPage.LastOpenBlock();
            PDFLayoutRegion containerRegion = containerBlock.CurrentRegion;

            if (containerBlock.CurrentRegion.HasOpenItem)
                containerBlock.CurrentRegion.CloseCurrentItem();

            //Do we have space
            if (containerRegion.AvailableHeight <= 0 || (position.HasHeight && containerRegion.AvailableHeight < position.Height))
            {
                if (this.MoveToNextRegion(ref containerRegion, ref containerBlock, out newPage) == false)
                {
                    this.ContinueLayout = false;
                    return;
                }

            }

            CurrentBlock = containerBlock.BeginNewContainerBlock(this.Component, this, this.FullStyle, position.PositionMode);
            CurrentBlock.BlockRepeatIndex = 0;
           
            //Layout of the block should start at 0, height.
            //TODO: Modify

            PDFRect avail = containerBlock.CurrentRegion.UnusedBounds;
            if (position.HasWidth)
                avail.Width = position.Width;
            else if (position.FillWidth)
                avail.Width -= position.Margins.Left + position.Margins.Right;
            

            if (position.HasHeight)
                avail.Height = position.Height;

            PDFRect total = new PDFRect(PDFPoint.Empty, avail.Size);//avail
            

            PDFUnit alley;
            int columncount = this.GetBlockColumnCount(out alley);
            total.Location = new PDFPoint(0, containerBlock.CurrentRegion.Height); 
            CurrentBlock.InitRegions(total, position, columncount, alley);

            this.DoLayoutChildren();

            PDFLayoutBlock block = this.CurrentBlock;
            //re-retrieve our container block and region
            //it could have changed whilst laying out other regions / pages.
            containerBlock = block.Parent as PDFLayoutBlock;
            if (block.IsClosed == false)
                block.Close();
            
            containerRegion = containerBlock.CurrentRegion;

            //ADDED to support blocks flowing to the next region or page
            PDFUnit vspace = containerRegion.TotalBounds.Height - containerRegion.UsedSize.Height;
            PDFUnit req = block.Height;

            bool canfitvertical = req < vspace;
            

            if (!canfitvertical && containerRegion.Contents.Count > 1)
            {
                PDFLayoutRegion prev = containerRegion;
                if (this.MoveToNextRegion(ref containerRegion, ref containerBlock, out newPage))
                {
                    prev.Contents.Remove(block);
                    containerRegion.AddExistingItem(block);
                }
                else
                {
                    //TODO: Check the truncate or overflow status
                    this.ContinueLayout = false;
                }
            }
            else
            {
                //Region has not updated its size as the block was directly closed.
                containerRegion.AddToSize(block);
            }
        }


        
        
    }
}
