﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Components
{
    /// <summary>
    /// Represents and encapsulates a date value that can be rendered to a PDF
    /// </summary>
    [PDFParsableComponent("Date")]
    public class PDFDate : PDFTextBase
    {
        //
        // properties
        //

        #region public DateTime Value { get; set; }

        /// <summary>
        /// The bindable date value. Default is the current date time
        /// </summary>
        [PDFAttribute("value")]
        public DateTime Value { get; set; }

        #endregion

        #region public string DateFormat {get;set;}

        /// <summary>
        /// Gets or sets the bindable date format string for conversion from value to a string
        /// </summary>
        [PDFAttribute("date-format", Const.PDFStylesNamespace)]
        public string DateFormat
        {
            get
            {
                Scryber.Styles.PDFTextStyle style;
                if (this.Style.TryGetTextStyle(out style))
                    return style.DateFormat;
                else
                    return string.Empty;
            }
            set
            {
                if (string.IsNullOrEmpty(value))
                    this.Style.Text.RemoveDateFormat();
                else
                    this.Style.Text.DateFormat = value;
            }
        }

        #endregion

        //
        // .ctor
        //

        #region public PDFDate()

        public PDFDate()
            : this(PDFObjectTypes.DateComp)
        {
        }

        #endregion

        #region protected PDFDate(PDFObjectType type)

        protected PDFDate(PDFObjectType type)
            : base(type)
        {
            this.Value = DateTime.Now;
        }

        #endregion

        //
        // methods
        //

        #region protected override Text.PDFTextReader CreateReader(PDFLayoutContext context, Styles.PDFStyle fullstyle)

        /// <summary>
        /// Overrides the base implementation to create a reader for the converted date value
        /// </summary>
        /// <param name="context"></param>
        /// <param name="fullstyle"></param>
        /// <returns></returns>
        protected override Text.PDFTextReader CreateReader(PDFLayoutContext context, Styles.PDFStyle fullstyle)
        {
            string val;

            Styles.PDFTextStyle text;
            if (fullstyle.TryGetTextStyle(out text) && !string.IsNullOrEmpty(text.DateFormat))
                val = this.Value.ToString(text.DateFormat);
            else
                val = this.Value.ToString();

            return Scryber.Text.PDFTextReader.Create(val, TextFormat.Plain, false);
        }

        #endregion
    }
}
