﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using Scryber.Styles;
using Scryber.Drawing;
namespace Scryber.Components
{
    [PDFParsableComponent("Line")]
    public class PDFLine : PDFShapeComponent
    {

        public PDFLine()
            : base(PDFObjectTypes.ShapeLine)
        {
        }


        protected override PDFStyle GetBaseStyle()
        {
            PDFStyle inherited = base.GetBaseStyle();
            inherited.Stroke.Width = new PDFUnit(1, PageUnits.Points);
            inherited.Stroke.Color = PDFColors.Black;
            inherited.Position.PositionMode = Scryber.Drawing.PositionMode.Block;

            return inherited;
        }


        protected override PDFGraphicsPath CreatePath(PDFSize available, PDFStyle fullstyle)
        {
            PDFPositionStyle pos;

            PDFPoint start = PDFPoint.Empty;
            PDFPoint end = PDFPoint.Empty;
            
            if (fullstyle.TryGetPosition(out pos))
            {
                if (pos.IsDefined(StyleKeys.XAttr))
                    start.X = pos.X;
                if (pos.IsDefined(StyleKeys.YAttr))
                    start.Y = pos.Y;

                if (pos.IsDefined(StyleKeys.WidthAttr))
                {
                    end.X = pos.Width - start.X;

                    if (pos.IsDefined(StyleKeys.HeightAttr))
                        end.Y = pos.Height - start.Y;
                    else // no hight so this is a horizontal line
                        end.Y = start.Y;
                }
                else if (pos.FullWidth == true)
                {
                    end.X = available.Width - start.X;

                    if (pos.IsDefined(StyleKeys.HeightAttr))
                        end.Y = pos.Height - start.Y;
                    else // no hight so this is a horizontal line
                        end.Y = start.Y;

                }
                //no width so this is a vertical line
                else if (pos.IsDefined(StyleKeys.HeightAttr))
                {
                    end.Y = pos.Height - start.Y;
                    end.X = start.X;
                }
                else //default is a horizontal line
                {
                    end.X = available.Width - start.X;
                    end.Y = start.Y;
                }
            }
            else
            {
                end.X = available.Width;
                end.Y = start.Y;
            }

            PDFGraphicsPath path = new PDFGraphicsPath();
            path.MoveTo(start);
            path.LineTo(end);

            return path;
        }
        
    }
}
