﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Components
{
    /// <summary>
    /// Represents a numeric value and converts to a string in the content
    /// </summary>
    [PDFParsableComponent("Number")]
    public class PDFNumber : PDFTextBase
    {
        //
        // properties
        //

        #region public double Value { get; set; }

        /// <summary>
        /// Gets or sets the bindable double value for this number
        /// </summary>
        [PDFAttribute("value")]
        public double Value { get; set; }

        #endregion

        #region public string NumberFormat {get;set;}

        /// <summary>
        /// Gets or sets the format to use to convert the number value to a string
        /// </summary>
        [PDFAttribute("number-format", Const.PDFStylesNamespace)]
        public string NumberFormat
        {
            get
            {
                Scryber.Styles.PDFTextStyle style;
                if (this.Style.TryGetTextStyle(out style))
                    return style.NumberFormat;
                else
                    return string.Empty;
            }
            set
            {
                if (string.IsNullOrEmpty(value))
                    this.Style.Text.RemoveNumberFormat();
                else
                    this.Style.Text.NumberFormat = value;
            }
        }

        #endregion

        //
        // .ctor
        //

        #region public PDFNumber()

        public PDFNumber()
            : this(PDFObjectTypes.NumberComp)
        {
        }

        #endregion

        #region protected PDFNumber(PDFObjectType type)

        protected PDFNumber(PDFObjectType type)
            : base(type)
        {
        }

        #endregion


        //
        // methods
        //

        #region protected override Text.PDFTextReader CreateReader(PDFLayoutContext context, Styles.PDFStyle fullstyle)

        /// <summary>
        /// overrides the default implementation to create a text reader for the Number in the correct format
        /// </summary>
        /// <param name="context"></param>
        /// <param name="fullstyle"></param>
        /// <returns></returns>
        protected override Text.PDFTextReader CreateReader(PDFLayoutContext context, Styles.PDFStyle fullstyle)
        {
            string val;

            Styles.PDFTextStyle text;
            if (fullstyle.TryGetTextStyle(out text) && !string.IsNullOrEmpty(text.NumberFormat))
                val = this.Value.ToString(text.NumberFormat);
            else
                val = this.Value.ToString();
            return Scryber.Text.PDFTextReader.Create(val, TextFormat.Plain, false);
        }

        #endregion
    }
}
