﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Components
{

    /// <summary>
    /// Base class for the page header and footer
    /// </summary>
    public class PDFPageAdornment : PDFLayoutTemplateComponent
    {
        public PDFPageAdornment(PDFObjectType type)
            : base(type)
        {
        }

        protected override Styles.PDFStyle GetBaseStyle()
        {
            Styles.PDFStyle style = base.GetBaseStyle();
            style.Position.FullWidth = true;

            return style;
        }
    }


    /// <summary>
    /// The PDFPageHeader is a template container for the PDFPage and PDFSection. 
    /// It is a parsable PDF component, but will only act like a block in the content of a page
    /// </summary>
    [PDFParsableComponent("PageHeader")]
    public class PDFPageHeader : PDFPageAdornment
    {
        public PDFPageHeader() : base(PDFObjectTypes.PageHeader) { }

    }


    /// <summary>
    /// The PDFPageFooter is a template container for the PDFPage and PDFSection. It is a non-parsable PDF component
    /// </summary>
    [PDFParsableComponent("PageFooter")]
    public class PDFPageFooter : PDFPageAdornment
    {
        public PDFPageFooter() : base(PDFObjectTypes.PageFooter) { }
    }
}
