﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using Scryber.Styles;
using Scryber.Resources;
using Scryber.Text;
using Scryber.Drawing;
using Scryber.Native;

namespace Scryber.Components
{
    public abstract class PDFTextBase : PDFVisualComponent, IPDFTextComponent
    {

        

        #region protected virtual string BaseText {get;set;}

        private string _text;

        protected virtual string BaseText
        {
            get { return _text; }
            set { _text = value; this.ResetTextBlock(); }
        }

        #endregion

        #region protected PDFTextBlock TextBlock {get;set;} + Interface implementation

        private PDFTextLayout _textops = null;

        protected PDFTextLayout TextLayout
        {
            get { return this._textops; }
            set { this._textops = value; }
        }

        PDFTextLayout IPDFTextComponent.TextLayout
        {
            get { return this.TextLayout; }
            set { this.TextLayout = value; }
        }

        #endregion

        #region protected .ctor(type)

        protected PDFTextBase(PDFObjectType type)
            : base(type)
        {
        }

        #endregion

        #region protected virtual void ResetTextBlock() + Interface implmentation

        protected virtual void ResetTextBlock()
        {
            _textops = null;
        }

        void IPDFTextComponent.ResetTextBlock()
        {
            this.ResetTextBlock();
        }

        #endregion

        #region protected virtual PDFTextReader CreateReader() + Interface implementation

        protected virtual PDFTextReader CreateReader(PDFLayoutContext context, PDFStyle fullstyle)
        {
            PDFTextStyle txt;
            TextFormat format = TextFormat.Plain;
            bool preserveWhitespace = false;
            if (fullstyle.TryGetTextStyle(out txt))
                preserveWhitespace = txt.PreserveWhitespace;

            if (this is IPDFTextLiteral)
                format = ((IPDFTextLiteral)this).ReaderFormat;
            
            return PDFTextReader.Create(this.BaseText, format, preserveWhitespace);
        }

        PDFTextReader IPDFTextComponent.CreateReader(PDFLayoutContext context, PDFStyle fullstyle)
        {
            return this.CreateReader(context, fullstyle);
        }

        #endregion

        protected override PDFStyle GetBaseStyle()
        {
            PDFStyle def = base.GetBaseStyle();
            def.Position.PositionMode = Scryber.Drawing.PositionMode.Inline;

            return def;
        }

        
    }
}
