﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;

namespace Scryber.Configuration
{
    /// <summary>
    /// 
    /// </summary>
    public static class ImagingConfiguration
    {
        private static string ImagingSectionPath = ScryberConfiguration.ScryberConfigGroupKey + "/imaging";


        #region public static bool IsFactoryRegisterd(string path, out IPDFImageDataFactory factory)

        /// <summary>
        /// Searches the configured image factories and returns true if an appropriate image factory was found for the specified path.
        /// </summary>
        /// <param name="path"></param>
        /// <param name="factory">If a match was found then this is set to the factory to use to create the image</param>
        /// <returns></returns>
        public static bool IsFactoryRegisterd(string path, out IPDFImageDataFactory factory)
        {
            object section = ConfigurationManager.GetSection(ImagingSectionPath);
            if (null == section)
            {
                factory = null;
                return false;
            }
            else if (section is ImagingConfigurationSection)
            {
                factory = ((ImagingConfigurationSection)section).GetRegisterdFactory(path);
                return null != factory;
            }
            else
            {
                string msg = String.Format(Errors.CannotCastObjectToType, section.GetType(), typeof(ImagingConfigurationSection));
                throw new ConfigurationErrorsException(msg);
            }
        }

        #endregion 
    }
}
