﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using Scryber.Drawing;

namespace Scryber
{
    /// <summary>
    /// A class that represents a full set of positioning data
    /// </summary>
    public class PDFPositionOptions
    {
        /// <summary>
        /// Gets or sets the position mode
        /// </summary>
        public PositionMode PositionMode { get; set; }

        /// <summary>
        /// Gets or sets the visibility of this content
        /// </summary>
        public Visibility Visibility { get; set; }

        /// <summary>
        /// Gets or sets the X (horizontal) offset
        /// </summary>
        public PDFUnit X { get; set; }

        /// <summary>
        /// Gets or sets the Y (vertical offset)
        /// </summary>
        public PDFUnit Y { get; set; }

        private PDFUnit _w;
        /// <summary>
        /// Gets or sets the Width
        /// </summary>
        public PDFUnit Width
        {
            get { return _w; }
            set { _w = value; }
        }

        /// <summary>
        /// Gets or sets the Height
        /// </summary>
        public PDFUnit Height { get; set; }

        /// <summary>
        /// Gets or sets if these options have a specific X value
        /// </summary>
        public bool HasX { get; set; }

        /// <summary>
        /// Gets or sets if these options have a specific Y value
        /// </summary>
        public bool HasY { get; set; }

        /// <summary>
        /// Gets or sets if these options have a specific width value
        /// </summary>
        public bool HasWidth { get; set; }

        /// <summary>
        /// Gets or sets if these options have a specific height value
        /// </summary>
        public bool HasHeight { get; set; }

        /// <summary>
        /// Gets or sets if these options specifiy that the copmonent these options refer to should fill the available horizontal space
        /// </summary>
        public bool FillWidth { get; set; }

        /// <summary>
        /// Gets or sets the overflow action to be taken when the available space is reached
        /// </summary>
        public OverflowAction OverflowAction { get; set; }

        /// <summary>
        /// Gets or sets the split options for child content
        /// </summary>
        public OverflowSplit OverflowSplit { get; set; }

        /// <summary>
        /// Gets or sets any margins
        /// </summary>
        public PDFThickness Margins { get; set; }

        /// <summary>
        /// Gets or sets any padding
        /// </summary>
        public PDFThickness Padding { get; set; }

        /// <summary>
        /// Gets or sets the vertical alignment on any child content
        /// </summary>
        public VerticalAlignment VAlign { get; set; }

        /// <summary>
        /// Gets or sets the horizontal alignment on any child content
        /// </summary>
        public HorizontalAlignment HAlign { get; set; }

        
        /// <summary>
        /// Creates a new default position options
        /// </summary>
        public PDFPositionOptions()
        {
            this.PositionMode = Drawing.PositionMode.Block;
            this.OverflowAction = Drawing.OverflowAction.NewPage;
            this.OverflowSplit = Drawing.OverflowSplit.Any;
        }


    }
}
