﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Text
{
    public class PDFEncoding
    {

        private System.Text.Encoding _sysEncoding;
        private byte[] _pre;

        /// <summary>
        /// Gets the bytes that should be written 
        /// before the start of a character string to identify the encoding
        /// </summary>
        public byte[] Prefix
        {
            get { return _pre; }
        }

        /// <summary>
        /// Returns true if this encoding has a prefix to be written
        /// </summary>
        public bool HasPrefix
        {
            get { return null != _pre && _pre.Length > 0; }
        }


        public PDFEncoding(System.Text.Encoding systemEncoding, byte[] prefix)
        {
            if (null == systemEncoding)
                throw new ArgumentNullException("systemEncoding");

            this._sysEncoding = systemEncoding;
            
            this._pre = prefix;
        }

        
        public byte[] GetBytes(string data)
        {
            return this._sysEncoding.GetBytes(data);
        }

        private static PDFEncoding _windows1252 = new PDFEncoding(System.Text.Encoding.GetEncoding("windows-1252"), null);
        private static PDFEncoding _unicodeBE = new PDFEncoding(System.Text.Encoding.GetEncoding("UTF-16BE"), null);
        private static PDFEncoding _macroman = new PDFEncoding(System.Text.Encoding.Default, null);
        private static PDFEncoding _pdfdoc = new PDFEncoding(System.Text.Encoding.ASCII, null);

        public static PDFEncoding WinAnsiEncoding
        {
            get { return _windows1252; }
        }

        public static PDFEncoding MacRomanEncoding
        {
            get { return _macroman; }
        }

        public static PDFEncoding PDFDocEncoding
        {
            get { return _pdfdoc; }
        }

        public static PDFEncoding UnicodeBigEndian
        {
            get { return _unicodeBE; }
        }

        public static PDFEncoding GetEncoding(FontEncoding fontencoding)
        {
            PDFEncoding enc;
            switch (fontencoding)
            {
                case FontEncoding.MacRomanEncoding:
                    enc = MacRomanEncoding;
                    break;
                case FontEncoding.WinAnsiEncoding:
                    enc = WinAnsiEncoding;
                    break;
                case FontEncoding.PDFDocEncoding:
                    enc = PDFDocEncoding;
                    break;
                case FontEncoding.UnicodeEncoding:
                    enc = WinAnsiEncoding;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("fontenncoding");
            }
            return enc;
        }

        public static PDFEncoding GetEncoding(string sysname)
        {
            System.Text.Encoding found = System.Text.Encoding.GetEncoding(sysname);
            if (null == found)
                throw new ArgumentOutOfRangeException("sysname");
            return new PDFEncoding(found, null);
        }
    }
}
