﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Text
{
    /// <summary>
    /// Tokenizes the text EXACTLY as it was passed in to the reader. No whitespace or carriage returns are removed.
    /// </summary>
    public class PDFPreformattedTextReader : PDFTextReader
    {

        private string _srctext;
        private string[] _lines;
        private int _index;
        private PDFTextOp _curr;

        public override PDFTextOp Value
        {
            get
            {
                if (_curr == null)
                {
                    if (_index < 0)
                        throw new ArgumentOutOfRangeException(Errors.CannotAccessStreamWithoutRead);
                    if (_index >= _lines.Length)
                        throw new ArgumentOutOfRangeException(Errors.CannotReadPastTheEOF);
                    string s = this._lines[_index];
                    if (s == "\n")
                        _curr = new PDFTextNewLineOp();
                    else
                        _curr = new PDFTextDrawOp(s);
                }
                return _curr;
            }
        }

        public override int Length
        {
            get { return _srctext.Length; }
        }

        public override bool EOF
        {
            get { return _index >= _lines.Length; }
        }

        public PDFPreformattedTextReader(string text)
            : base()
        {
            this._srctext = text;
            this._lines = null == _srctext ? new string[] { } : _srctext.Split('\n');
            if (_lines.Length > 0)
            {
                List<string> act = new List<string>();
                foreach (string s in this._lines)
                {
                    act.Add(s);
                    act.Add("\n");
                }
                act.RemoveAt(act.Count - 1);
                this._lines = act.ToArray();
            }
            this.ResetTextMarkers();
        }


        public override bool Read()
        {
            _index++;
            _curr = null;
            return _index < _lines.Length;
        }

        protected override void ResetTextMarkers()
        {
            this._index = -1;
            this._curr = null;
        }
    }
}
