﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Text
{
    public class PDFXMLFragmentReader : Scryber.Text.PDFTextReader
    {

        private const int StartIndex = -1;

        private int _index;
        private List<Scryber.Text.PDFTextOp> _all;
        private Exception _error;

        public override bool EOF
        {
            get
            {
                RethrowAnyError();
                return _all.Count <= _index;
            }
        }

        public override int Length
        {
            get
            {
                RethrowAnyError();
                return _all.Count;
            }
        }

        public override Scryber.Text.PDFTextOp Value
        {
            get
            {
                RethrowAnyError();
                if (EOF)
                    throw new InvalidOperationException("This reader has passed the end of the data");
                else if (_index <= StartIndex)
                    throw new InvalidOperationException("Call Read() before attempting to extract any value");

                return _all[_index];
            }
        }

        //
        // ctor(s)
        //

        public PDFXMLFragmentReader(string text, bool preserveWhitespace)
        {
            this.InitWithText(text, preserveWhitespace);
        }


        //
        // public / protected methods
        //

        public override bool Read()
        {
            _index++;
            return !EOF;
        }

        protected override void ResetTextMarkers()
        {
            this._index = StartIndex;
        }


        //
        // private implementation
        //


        /// <summary>
        /// Initializes the XMLFragmentReader with a string
        /// </summary>
        /// <param name="text"></param>
        private void InitWithText(string text, bool preserveWhitespace)
        {
            List<Scryber.Text.PDFTextOp> all = null;

            try
            {
                PDFXMLFragmentParser parser = new PDFXMLFragmentParser();
                all = parser.Parse(text, preserveWhitespace);
            }
            catch (Exception ex)
            {
                _error = ex;
                throw new PDFXmlFormatException("Could not parse the XML text in the provided string", ex);
            }

            //set the instance variable in pre-read state
            _all = all;
            _index = -1;
        }



        /// <summary>
        /// Checks the error instance variable, and if set re-throws wrapped in a new exception
        /// </summary>
        private void RethrowAnyError()
        {
            if (null != _error)
                throw new PDFXmlFormatException("Could not parse the XML text in the provided string", _error);
        }
    }
}
