﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Text;
using Scryber.Drawing;

namespace Scryber.Text
{
    /// <summary>
    /// Represents a set of characters on a single line
    /// </summary>
    public class TextSpan : TextData
    {
        #region public int Length {get;}

        /// <summary>
        /// Gets the length of characters in this span
        /// </summary>
        public int Length
        {
            get 
            {
                if (string.IsNullOrEmpty(this._chars))
                    return 0;
                else
                    return this._chars.Length;
            }
        }

        #endregion

        #region public string Characters {get;set;}

        private string _chars;

        /// <summary>
        /// Gets or sets characters in this span
        /// </summary>
        public string Characters
        {
            get { return _chars; }
            set { _chars = value; }
        }

        #endregion

        //.ctor(s)

        #region public TextSpan()

        /// <summary>
        /// Creates a new empty TextSpan
        /// </summary>
        public TextSpan()
            : this(null)
        {
        }

        #endregion

        #region public TextSpan(string chars)

        /// <summary>
        /// Creates a new TextSpan with the specified characters within the span
        /// </summary>
        /// <param name="chars"></param>
        public TextSpan(string chars)
            : base()
        {
            this._chars = chars;
        }

        #endregion

        public override string ToString()
        {
            return "/(" + this.Characters + ")";
        }
    }
}
