﻿/*  Copyright 2012 PerceiveIT Limited
 *  This file is part of the Scryber library.
 *
 *  You can redistribute Scryber and/or modify 
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Scryber is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 * 
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Scryber source code in the COPYING.txt file.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Scryber.Text
{
    /// <summary>
    /// Represents a style change in a text block
    /// </summary>
    public class TextBeginStyle : TextData
    {
        private string _identifier;

        #region public override double Width, public override double Height - return 0;

        public override double Width
        {
            get
            {
                return 0.0;
            }
            set
            {
                throw new NotSupportedException("Cannot set the width or height of a TextStyle entry");
            }
        }

        public override double Height
        {
            get
            {
                return 0.0;
            }
            set
            {
                throw new NotSupportedException("Cannot set the width or height of a TextStyle entry");
            }
        }

        #endregion

        #region public PDFTextRenderOptions StyleOptions {get;set;}

        private PDFTextRenderOptions _opts;

        /// <summary>
        /// Gets or sets the render options for this style change
        /// </summary>
        public PDFTextRenderOptions RenderOptions
        {
            get { return _opts; }
            set { _opts = value; }
        }

        #endregion


        // .ctor

        #region public TextStyle(PDFTextRenderOptions options)

        /// <summary>
        /// Creates a new TextStyle with the specified text rendering options
        /// </summary>
        /// <param name="options"></param>
        public TextBeginStyle(PDFTextRenderOptions options, string identifier)
        {
            this._opts = options;
            this._identifier = identifier;
        }

        #endregion

        public override string ToString()
        {
            return "/" + _identifier;
        }
    }

    public class TextEndStyle : TextData
    {
        private string _identifier;

        #region public override double Width, public override double Height - return 0;

        public override double Width
        {
            get
            {
                return 0.0;
            }
            set
            {
                throw new NotSupportedException("Cannot set the width or height of a TextStyle entry");
            }
        }

        public override double Height
        {
            get
            {
                return 0.0;
            }
            set
            {
                throw new NotSupportedException("Cannot set the width or height of a TextStyle entry");
            }
        }

        #endregion


        #region public PDFTextRenderOptions StyleOptions {get;set;}

        private PDFTextRenderOptions _opts;

        /// <summary>
        /// Gets or sets the render options for this style change
        /// </summary>
        public PDFTextRenderOptions RenderOptions
        {
            get { return _opts; }
            set { _opts = value; }
        }

        #endregion

        public TextEndStyle(PDFTextRenderOptions opts, string identifier)
            : base()
        {
            this._opts = opts;
            this._identifier = identifier;
        }

        public override string ToString()
        {
            return "/!" + _identifier;
        }
        
    }
}
