﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Scryber.Drawing;

namespace Scryber.Styles
{
    [PDFParsableComponent("Columns")]
    public class PDFColumnsStyle : PDFStyleItem
    {
        public static readonly PDFUnit DefaultAlleyWidth = new PDFUnit(10, PageUnits.Points);
        public const bool DefaultAutoFlow = true;

        public PDFColumnsStyle()
            : this(PDFObjectTypes.StyleColumns, NOT_INHERITED)
        {
        }

        public PDFColumnsStyle(PDFObjectType type, bool inherited)
            : base(type, inherited)
        {
        }

        //
        // style properties
        //

        #region public int ColumnCount {get;set;} + RemoveColumnCount

        /// <summary>
        /// Gets or sets the number of columns
        /// </summary>
        [PDFAttribute("count")]
        public int ColumnCount
        {
            get
            {
                int count;
                if (this.GetIntegerValue(StyleKeys.ColumnCountAttr, false, out count))
                    return count;
                else
                    return 1;
            }
            set
            {
                this.SetValue(StyleKeys.ColumnCountAttr, value);
            }
        }

        public void RemoveColumnCount()
        {
            this.Remove(StyleKeys.ColumnCountAttr);
        }

        #endregion

        #region PDFUnit AlleyWidth {get;set;} RemoveAlleyWidth()

        /// <summary>
        /// Gets or sets the width of the column alley - space between the columns
        /// </summary>
        [PDFAttribute("alley-width")]
        public PDFUnit AlleyWidth
        {
            get
            {
                PDFUnit width;
                if (this.GetUnitValue(StyleKeys.AlleyWidthAttr, false, out width))
                    return width;
                else
                    return DefaultAlleyWidth;
            }
            set
            {
                this.SetValue(StyleKeys.AlleyWidthAttr, value);
            }
        }

        public void RemoveAlleyWidth()
        {
            this.Remove(StyleKeys.AlleyWidthAttr);
        }

        #endregion

        /// <summary>
        /// If true then the content will fill one available column and then 
        /// automatically flow onto the next column. If false then the content will only flow 
        /// onto the next column after an explicit instruction (e.g. pdf:ColumnBreak). 
        /// The default is true.
        /// </summary>
        [PDFAttribute("auto-flow")]
        public bool AutoFlow
        {
            get
            {
                bool auto;
                if (this.GetBoolValue(StyleKeys.AutoFlow, false, out auto))
                    return auto;
                else
                    return DefaultAutoFlow;
            }
            set
            {
                this.SetValue(StyleKeys.AutoFlow, value);
            }
        }
    }

}
