﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Scryber.Drawing;

namespace Scryber.Styles
{

    [PDFParsableComponent("Badge")]
    public class PDFScryberBadgeStyle : PDFStyleItem
    {

        private static readonly PDFUnit DefaultXOffset = new PDFUnit(10,PageUnits.Millimeters);
        private static readonly PDFUnit DefaultYOffset = new PDFUnit(10,PageUnits.Millimeters);
        private const Reversible DefaultDisplayOption = Reversible.WhiteOnBlack;
        

        #region ctor(s)

        public PDFScryberBadgeStyle()
            : this(PDFObjectTypes.StyleBadge, false)
        {
        }

        protected PDFScryberBadgeStyle(PDFObjectType type, bool inherited)
            : base(type, inherited)
        {
        }

        #endregion


        [PDFAttribute("width")]
        public PDFUnit Width
        {
            get
            {
                PDFUnit val;
                if (this.GetUnitValue(StyleKeys.WidthAttr, false, out val))
                    return val;
                else
                    return PDFUnit.Empty;
            }
            set
            {
                if (value.PointsValue < 10)
                    throw new ArgumentOutOfRangeException("Cannot set the badge size to less than 10pt");

                this.SetValue(StyleKeys.WidthAttr, value);
            }
        }


        [PDFAttribute("height")]
        public PDFUnit Height
        {
            get
            {
                PDFUnit val;
                if (this.GetUnitValue(StyleKeys.HeightAttr, false, out val))
                    return val;
                else
                    return PDFUnit.Empty;
            }
            set
            {
                if (value.PointsValue < 10)
                    throw new ArgumentOutOfRangeException("Cannot set the badge size to less than 10pt");

                this.SetValue(StyleKeys.HeightAttr, value);
            }
        }

        [PDFAttribute("corner")]
        public Corner Corner
        {
            get
            {
                object val;
                if (this.GetEnumValue(StyleKeys.CornerAttr, typeof(Corner), false, out val))
                    return (Corner)val;
                else
                    return Corner.BottomRight;
            }
            set
            {
                this.SetValue(StyleKeys.CornerAttr, value);
            }
        }

        [PDFAttribute("display")]
        public Reversible DisplayOption
        {
            get
            {
                object val;
                if (this.GetEnumValue(StyleKeys.DisplayOptionAttr, typeof(Reversible), false, out val))
                    return (Reversible)val;
                else
                    return DefaultDisplayOption;
            }
            set
            {
                this.SetValue(StyleKeys.DisplayOptionAttr, value);
            }

        }


        [PDFAttribute("x-offset")]
        public PDFUnit XOffset
        {
            get
            {
                PDFUnit val;
                if (this.GetUnitValue(StyleKeys.XOffsetAttr, false, out val))
                    return val;
                else
                    return DefaultXOffset;
            }
            set
            {
                this.SetValue(StyleKeys.XOffsetAttr, value);
            }
        }

        [PDFAttribute("y-offset")]
        public PDFUnit YOffset
        {
            get
            {
                PDFUnit val;
                if (this.GetUnitValue(StyleKeys.YOffsetAttr, false, out val))
                    return val;
                else
                    return DefaultYOffset;
            }
            set
            {
                this.SetValue(StyleKeys.YOffsetAttr, value);
            }
        }
    }
}
